libata: pata_pdc2027x PLL input clock fix
[sfrench/cifs-2.6.git] / ipc / shm.c
index eb57e22543049f2bb157505458fc90308a797543..0852f206d8951cb4f53071b36e159ae0332e5093 100644 (file)
--- a/ipc/shm.c
+++ b/ipc/shm.c
@@ -226,8 +226,8 @@ static void shm_close(struct vm_area_struct *vma)
        mutex_unlock(&shm_ids(ns).mutex);
 }
 
-struct page *shm_nopage(struct vm_area_struct *vma, unsigned long address,
-                       int *type)
+static struct page *shm_nopage(struct vm_area_struct *vma,
+                              unsigned long address, int *type)
 {
        struct file *file = vma->vm_file;
        struct shm_file_data *sfd = shm_file_data(file);
@@ -254,8 +254,10 @@ struct mempolicy *shm_get_policy(struct vm_area_struct *vma, unsigned long addr)
 
        if (sfd->vm_ops->get_policy)
                pol = sfd->vm_ops->get_policy(vma, addr);
-       else
+       else if (vma->vm_policy)
                pol = vma->vm_policy;
+       else
+               pol = current->mempolicy;
        return pol;
 }
 #endif
@@ -285,21 +287,41 @@ static int shm_release(struct inode *ino, struct file *file)
        return 0;
 }
 
-#ifndef CONFIG_MMU
+static int shm_fsync(struct file *file, struct dentry *dentry, int datasync)
+{
+       int (*fsync) (struct file *, struct dentry *, int datasync);
+       struct shm_file_data *sfd = shm_file_data(file);
+       int ret = -EINVAL;
+
+       fsync = sfd->file->f_op->fsync;
+       if (fsync)
+               ret = fsync(sfd->file, sfd->file->f_path.dentry, datasync);
+       return ret;
+}
+
 static unsigned long shm_get_unmapped_area(struct file *file,
        unsigned long addr, unsigned long len, unsigned long pgoff,
        unsigned long flags)
 {
        struct shm_file_data *sfd = shm_file_data(file);
-       return sfd->file->f_op->get_unmapped_area(sfd->file, addr, len, pgoff,
-                                                       flags);
+       return get_unmapped_area(sfd->file, addr, len, pgoff, flags);
+}
+
+int is_file_shm_hugepages(struct file *file)
+{
+       int ret = 0;
+
+       if (file->f_op == &shm_file_operations) {
+               struct shm_file_data *sfd;
+               sfd = shm_file_data(file);
+               ret = is_file_hugepages(sfd->file);
+       }
+       return ret;
 }
-#else
-#define shm_get_unmapped_area NULL
-#endif
 
 static const struct file_operations shm_file_operations = {
        .mmap           = shm_mmap,
+       .fsync          = shm_fsync,
        .release        = shm_release,
        .get_unmapped_area      = shm_get_unmapped_area,
 };
@@ -344,9 +366,10 @@ static int newseg (struct ipc_namespace *ns, key_t key, int shmflg, size_t size)
                return error;
        }
 
+       sprintf (name, "SYSV%08x", key);
        if (shmflg & SHM_HUGETLB) {
-               /* hugetlb_zero_setup takes care of mlock user accounting */
-               file = hugetlb_zero_setup(size);
+               /* hugetlb_file_setup takes care of mlock user accounting */
+               file = hugetlb_file_setup(name, size);
                shp->mlock_user = current->user;
        } else {
                int acctflag = VM_ACCOUNT;
@@ -357,7 +380,6 @@ static int newseg (struct ipc_namespace *ns, key_t key, int shmflg, size_t size)
                if  ((shmflg & SHM_NORESERVE) &&
                                sysctl_overcommit_memory != OVERCOMMIT_NEVER)
                        acctflag = 0;
-               sprintf (name, "SYSV%08x", key);
                file = shmem_file_setup(name, size, acctflag);
        }
        error = PTR_ERR(file);
@@ -377,6 +399,11 @@ static int newseg (struct ipc_namespace *ns, key_t key, int shmflg, size_t size)
        shp->shm_nattch = 0;
        shp->id = shm_buildid(ns, id, shp->shm_perm.seq);
        shp->shm_file = file;
+       /*
+        * shmid gets reported as "inode#" in /proc/pid/maps.
+        * proc-ps tools use this. Changing this will break them.
+        */
+       file->f_dentry->d_inode->i_ino = shp->id;
 
        ns->shm_tot += numpages;
        shm_unlock(shp);